@@ -95,6 +95,8 @@ static char *std_principal_obj_classes[] = {
 
 #define STD_PRINCIPAL_OBJ_CLASSES_SIZE (sizeof(std_principal_obj_classes) / sizeof(char *) - 1)
 
+#define DEFAULT_TL_DATA_CONTENT "\x00\x00\x00\x00principal@UNINITIALIZED"
+
 static int ipadb_ldap_attr_to_tl_data(LDAP *lcontext, LDAPMessage *le,
                                       char *attrname,
                                       krb5_tl_data **result, int *num)
@@ -163,7 +165,7 @@ done:
 static krb5_error_code ipadb_set_tl_data(krb5_db_entry *entry,
                                          krb5_int16 type,
                                          krb5_ui_2 length,
-                                         krb5_octet *data)
+                                         const krb5_octet *data)
 {
     krb5_error_code kerr;
     krb5_tl_data *new_td = NULL;
@@ -324,17 +326,18 @@ static enum ipadb_user_auth ipadb_get_user_auth(struct ipadb_context *ipactx,
     enum ipadb_user_auth ua = IPADB_USER_AUTH_NONE;
     const struct ipadb_global_config *gcfg = NULL;
 
-    /* Get the user's user_auth settings. */
-    ipadb_parse_user_auth(ipactx->lcontext, lentry, &ua);
-
     /* Get the global user_auth settings. */
     gcfg = ipadb_get_global_config(ipactx);
     if (gcfg != NULL)
         gua = gcfg->user_auth;
 
-    /* If the disabled flag is set, ignore everything else. */
-    if ((ua | gua) & IPADB_USER_AUTH_DISABLED)
-        return IPADB_USER_AUTH_DISABLED;
+    /* Get the user's user_auth settings if not disabled. */
+    if ((gua & IPADB_USER_AUTH_DISABLED) == 0)
+        ipadb_parse_user_auth(ipactx->lcontext, lentry, &ua);
+
+    /* Filter out the disabled flag. */
+    gua &= ~IPADB_USER_AUTH_DISABLED;
+    ua &= ~IPADB_USER_AUTH_DISABLED;
 
     /* Determine which user_auth policy is active: user or global. */
     if (ua == IPADB_USER_AUTH_NONE)
@@ -348,6 +351,83 @@ static enum ipadb_user_auth ipadb_get_user_auth(struct ipadb_context *ipactx,
     return ua;
 }
 
+#define OSA_ADB_PRINC_VERSION_1  0x12345C01
+/* The XDR encoding of OSA_PRINC_ENC is as follows:
+    version:        int (signed 32 bit integer)
+    name:           nullstring (null terminated variable string)
+    aux_attributes: long (signed 32 bit integer)
+    old_key_next:   u_int (unsigned 32 bit integer)
+    adm_hist_kvno:  u_char (unisgned char)
+    old_keys:       array of keys, we do not care so alway u_int of 0
+*/
+#define OSA_PRINC_ENC_BASE_SIZE 20
+
+static krb5_error_code ipadb_policydn_to_kdam_tl_data(const char *policydn,
+                                                      krb5_db_entry *entry)
+{
+    krb5_error_code kerr;
+    uint32_t tmp;
+    char *policy_name = NULL;
+    char *p;
+    uint8_t *buf = NULL;
+    size_t buf_len;
+    int slen;
+    int plen;
+    int cur;
+
+    /* policy objects must use cn as the RDN */
+    if (strncmp(policydn, "cn=", 3) != 0) {
+        return KRB5_KDB_INTERNAL_ERROR;
+    }
+
+    /* Should we try to consider the case where a ',' is part of the polict
+     * name ? */
+    policy_name = strdup(&policydn[3]);
+    if (!policy_name) {
+        kerr = ENOMEM;
+        goto done;
+    }
+    p = strchr(policy_name, ',');
+    if (p) *p = '\0';
+
+    /* Now we open code a basic KRB5_TL_KADM_DATA which is a XDR encoded
+     * structure in MIT code */
+
+    slen = strlen(policy_name) + 1;
+    /* A xdr varstring is preceeded by a 32bit len field and is always 32
+     * bit aligned */
+    plen = slen + 4;
+    plen = (((plen + 3) / 4) * 4);
+
+    buf_len = OSA_PRINC_ENC_BASE_SIZE + plen;
+    buf = calloc(1, buf_len);
+    if (!buf) {
+        kerr = ENOMEM;
+        goto done;
+    }
+
+    /* version */
+    cur = 0;
+    tmp = htobe32(OSA_ADB_PRINC_VERSION_1);
+    memcpy(&buf[cur], &tmp, 4);
+    cur += 4;
+
+    /* name */
+    tmp = htobe32(slen);
+    memcpy(&buf[cur], &tmp, 4);
+    memcpy(&buf[cur + 4], policy_name, slen);
+    cur += plen;
+
+    /* All the other fileds are left empty */
+
+    kerr = ipadb_set_tl_data(entry, KRB5_TL_KADM_DATA, buf_len, buf);
+
+done:
+    free(policy_name);
+    free(buf);
+    return kerr;
+}
+
 static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
                                               char *principal,
                                               LDAPMessage *lentry,
@@ -516,7 +596,15 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
     case 0:
         entry->tl_data = res_tl_data;
         entry->n_tl_data = result;
+        break;
     case ENOENT:
+        /* The kadmin utility expects always at least KRB5_TL_MOD_PRINC tl_data
+         * to be available. So if krbExtraData is missing (may happen when a
+         * user is created but no password has been set yet) then add a default
+         * one. */
+        kerr = ipadb_set_tl_data(entry, KRB5_TL_MOD_PRINC,
+                                 sizeof(DEFAULT_TL_DATA_CONTENT),
+                                 (const krb5_octet *)DEFAULT_TL_DATA_CONTENT);
         break;
     default:
         kerr = KRB5_KDB_INTERNAL_ERROR;
@@ -616,6 +704,9 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
     }
     ied->pw_policy_dn = restring;
 
+    kerr = ipadb_policydn_to_kdam_tl_data(restring, entry);
+    if (kerr) goto done;
+
     ret = ipadb_ldap_attr_to_strlist(lcontext, lentry,
                                      "passwordHistory", &restrlist);
     if (ret != 0 && ret != ENOENT) {
@@ -830,6 +921,25 @@ static krb5_error_code ipadb_find_principal(krb5_context kcontext,
     return 0;
 }
 
+static krb5_flags maybe_require_preauth(struct ipadb_context *ipactx,
+                                        krb5_db_entry *entry)
+{
+    const struct ipadb_global_config *config;
+    struct ipadb_e_data *ied;
+
+    config = ipadb_get_global_config(ipactx);
+    if (config->disable_preauth_for_spns) {
+        ied = (struct ipadb_e_data *)entry->e_data;
+        if (ied && ied->ipa_user != true) {
+            /* not a user, assume SPN */
+            return 0;
+        }
+    }
+
+    /* By default require preauth for all principals */
+    return KRB5_KDB_REQUIRES_PRE_AUTH;
+}
+
 static krb5_error_code ipadb_fetch_tktpolicy(krb5_context kcontext,
                                              LDAPMessage *lentry,
                                              krb5_db_entry *entry,
@@ -900,7 +1010,7 @@ static krb5_error_code ipadb_fetch_tktpolicy(krb5_context kcontext,
                 if (ret == 0) {
                     entry->attributes |= result;
                 } else {
-                    entry->attributes |= KRB5_KDB_REQUIRES_PRE_AUTH;
+                    entry->attributes |= maybe_require_preauth(ipactx, entry);
                 }
             }
         }
@@ -916,7 +1026,7 @@ static krb5_error_code ipadb_fetch_tktpolicy(krb5_context kcontext,
             entry->max_renewable_life = 604800;
         }
         if (polmask & TKTFLAGS_BIT) {
-            entry->attributes |= KRB5_KDB_REQUIRES_PRE_AUTH;
+            entry->attributes |= maybe_require_preauth(ipactx, entry);
         }
 
         kerr = 0;
@@ -942,8 +1052,10 @@ krb5_error_code ipadb_get_principal(krb5_context kcontext,
     struct ipadb_context *ipactx;
     krb5_error_code kerr;
     char *principal = NULL;
+    char *trusted_realm = NULL;
     LDAPMessage *res = NULL;
     LDAPMessage *lentry;
+    krb5_db_entry *kentry = NULL;
     uint32_t pol;
 
     ipactx = ipadb_get_context(kcontext);
@@ -963,6 +1075,55 @@ krb5_error_code ipadb_get_principal(krb5_context kcontext,
 
     kerr = ipadb_find_principal(kcontext, flags, res, &principal, &lentry);
     if (kerr != 0) {
+        if ((kerr == KRB5_KDB_NOENTRY) &&
+            ((flags & (KRB5_KDB_FLAG_CANONICALIZE |
+                       KRB5_KDB_FLAG_CLIENT_REFERRALS_ONLY)) != 0)) {
+
+            /* First check if we got enterprise principal which looks like
+             * username\@enterprise_realm@REALM */
+            char *realm;
+            krb5_data *upn;
+
+            upn = krb5_princ_component(kcontext, search_for,
+                                       krb5_princ_size(kcontext, search_for) - 1);
+
+            if (upn == NULL) {
+                kerr = KRB5_KDB_NOENTRY;
+                goto done;
+            }
+
+            realm = memrchr(upn->data, '@', upn->length);
+            if (realm == NULL) {
+                kerr = KRB5_KDB_NOENTRY;
+                goto done;
+            }
+
+            /* skip '@' and use part after '@' as an enterprise realm for comparison */
+            realm++;
+
+            kerr = ipadb_is_princ_from_trusted_realm(kcontext,
+                                                     realm,
+                                                     upn->length - (realm - upn->data),
+                                                     &trusted_realm);
+            if (kerr == 0) {
+                kentry = calloc(1, sizeof(krb5_db_entry));
+                if (!kentry) {
+                    kerr = ENOMEM;
+                    goto done;
+                }
+                kerr = krb5_parse_name(kcontext, principal,
+                                       &kentry->princ);
+                if (kerr != 0) {
+                    goto done;
+                }
+
+                kerr = krb5_set_principal_realm(kcontext, kentry->princ, trusted_realm);
+                if (kerr != 0) {
+                    goto done;
+                }
+                *entry = kentry;
+            }
+        }
         goto done;
     }
 
@@ -979,6 +1140,10 @@ krb5_error_code ipadb_get_principal(krb5_context kcontext,
     }
 
 done:
+    free(trusted_realm);
+    if ((kerr != 0) && (kentry != NULL)) {
+        ipadb_free_principal(kcontext, kentry);
+    }
     ldap_msgfree(res);
     krb5_free_unparsed_name(kcontext, principal);
     return kerr;
