@@ -23,6 +23,7 @@ define([
         './jquery',
         './phases',
         './reg',
+        './rpc',
         './text',
         './association',
         './entity',
@@ -30,7 +31,7 @@ define([
         './rule',
         './search'
        ],
-            function(IPA, $, phases, reg, text) {
+            function(IPA, $, phases, reg, rpc, text) {
 
 var exp = IPA.sudo = {
     //priority of commands in details facet
@@ -61,12 +62,12 @@ var spec = {
                 {
                     name: 'disable',
                     label: '@i18n:buttons.disable',
-                    icon: 'disabled-icon'
+                    icon: 'fa-minus'
                 },
                 {
                     name: 'enable',
                     label: '@i18n:buttons.enable',
-                    icon: 'enabled-icon'
+                    icon: 'fa-check'
                 }
             ]
         },
@@ -80,7 +81,7 @@ var spec = {
                 'disable',
                 'delete'
             ],
-            header_actions: ['select_action', 'enable', 'disable', 'delete'],
+            header_actions: ['enable', 'disable', 'delete'],
             state: {
                 evaluators: [
                     {
@@ -226,6 +227,7 @@ return {
 };};
 
 /**
+ * @ignore
  * @param {Object} facet spec
  */
 var add_sudorule_details_facet_widgets = function (spec) {
@@ -240,6 +242,10 @@ var add_sudorule_details_facet_widgets = function (spec) {
             read_only: true,
             widget: 'general.cn'
         },
+        {
+            name: 'sudoorder',
+            widget: 'general.sudoorder'
+        },
         {
             $type: 'textarea',
             name: 'description',
@@ -249,13 +255,14 @@ var add_sudorule_details_facet_widgets = function (spec) {
 
     spec.widgets = [
         {
-            $type: 'details_table_section',
+            $type: 'details_section',
             name: 'general',
             label: '@i18n:details.general',
             widgets: [
                 {
                     name: 'cn'
                 },
+                'sudoorder',
                 {
                     $type: 'textarea',
                     name: 'description'
@@ -310,7 +317,7 @@ var add_sudorule_details_facet_widgets = function (spec) {
 
     spec.widgets.push(
         {
-            $factory: IPA.collapsible_section,
+            $factory: IPA.section,
             name: 'user',
             label: '@i18n:objects.sudorule.user',
             widgets: [
@@ -385,7 +392,7 @@ var add_sudorule_details_facet_widgets = function (spec) {
 
     spec.widgets.push(
     {
-            $factory: IPA.collapsible_section,
+            $factory: IPA.section,
             name: 'host',
             label: '@i18n:objects.sudorule.host',
             widgets: [
@@ -458,29 +465,23 @@ var add_sudorule_details_facet_widgets = function (spec) {
         {
             $type: 'rule_association_table',
             name: 'memberdenycmd_sudocmd',
-            widget: 'command.memberdenycmd_sudocmd',
+            widget: 'command.rule.memberdenycmd_sudocmd',
             priority: IPA.sudo.remove_method_priority
         },
         {
             $type: 'rule_association_table',
             name: 'memberdenycmd_sudocmdgroup',
-            widget: 'command.memberdenycmd_sudocmdgroup',
+            widget: 'command.rule.memberdenycmd_sudocmdgroup',
             priority: IPA.sudo.remove_method_priority
         }
     );
 
     spec.widgets.push(
         {
-            $factory: IPA.collapsible_section,
+            $factory: IPA.section,
             name: 'command',
             label: '@i18n:objects.sudorule.command',
             widgets: [
-                {
-                    $factory: IPA.header_widget,
-                    name: 'allow_header',
-                    text: '@i18n:objects.sudorule.allow',
-                    description: '@i18n:objects.sudorule.allow'
-                },
                 {
                     $factory: IPA.rule_details_widget,
                     name: 'rule',
@@ -497,9 +498,17 @@ var add_sudorule_details_facet_widgets = function (spec) {
                     ],
                     tables: [
                         { name: 'memberallowcmd_sudocmd' },
-                        { name: 'memberallowcmd_sudocmdgroup' }
+                        { name: 'memberallowcmd_sudocmdgroup' },
+                        { name: 'memberdenycmd_sudocmd' },
+                        { name: 'memberdenycmd_sudocmdgroup' }
                     ],
                     widgets: [
+                        {
+                            $factory: IPA.header_widget,
+                            name: 'allow_header',
+                            text: '@i18n:objects.sudorule.allow',
+                            description: '@i18n:objects.sudorule.allow'
+                        },
                         {
                             $type: 'rule_association_table',
                             id: 'sudorule-memberallowcmd_sudocmd',
@@ -517,32 +526,32 @@ var add_sudorule_details_facet_widgets = function (spec) {
                             remove_method: 'remove_allow_command',
                             add_title: '@i18n:association.add.memberallowcmd',
                             remove_title: '@i18n:association.remove.memberallowcmd'
+                        },
+                        {
+                            $factory: IPA.header_widget,
+                            name: 'deny_header',
+                            text: '@i18n:objects.sudorule.deny',
+                            description: '@i18n:objects.sudorule.deny'
+                        },
+                        {
+                            $type: 'rule_association_table',
+                            id: 'sudorule-memberdenycmd_sudocmd',
+                            name: 'memberdenycmd_sudocmd',
+                            add_method: 'add_deny_command',
+                            remove_method: 'remove_deny_command',
+                            add_title: '@i18n:association.add.memberdenycmd',
+                            remove_title: '@i18n:association.remove.memberdenycmd'
+                        },
+                        {
+                            $type: 'rule_association_table',
+                            id: 'sudorule-memberdenycmd_sudocmdgroup',
+                            name: 'memberdenycmd_sudocmdgroup',
+                            add_method: 'add_deny_command',
+                            remove_method: 'remove_deny_command',
+                            add_title: '@i18n:association.add.memberdenycmd',
+                            remove_title: '@i18n:association.remove.memberdenycmd'
                         }
                     ]
-                },
-                {
-                    $factory: IPA.header_widget,
-                    name: 'deny_header',
-                    text: '@i18n:objects.sudorule.deny',
-                    description: '@i18n:objects.sudorule.deny'
-                },
-                {
-                    $type: 'rule_association_table',
-                    id: 'sudorule-memberdenycmd_sudocmd',
-                    name: 'memberdenycmd_sudocmd',
-                    add_method: 'add_deny_command',
-                    remove_method: 'remove_deny_command',
-                    add_title: '@i18n:association.add.memberdenycmd',
-                    remove_title: '@i18n:association.remove.memberdenycmd'
-                },
-                {
-                    $type: 'rule_association_table',
-                    id: 'sudorule-memberdenycmd_sudocmdgroup',
-                    name: 'memberdenycmd_sudocmdgroup',
-                    add_method: 'add_deny_command',
-                    remove_method: 'remove_deny_command',
-                    add_title: '@i18n:association.add.memberdenycmd',
-                    remove_title: '@i18n:association.remove.memberdenycmd'
                 }
             ]
         }
@@ -587,7 +596,7 @@ var add_sudorule_details_facet_widgets = function (spec) {
 
     spec.widgets.push(
         {
-            $factory: IPA.collapsible_section,
+            $factory: IPA.section,
             name: 'runas',
             label: '@i18n:objects.sudorule.runas',
             widgets: [
@@ -680,7 +689,7 @@ IPA.sudo.options_section = function(spec) {
 
     spec = spec || {};
 
-    var that = IPA.collapsible_section(spec);
+    var that = IPA.section(spec);
 
     function setup_table(){
         that.table = IPA.table_widget({
@@ -701,30 +710,24 @@ IPA.sudo.options_section = function(spec) {
 
             that.table.table_create(container);
 
-            that.remove_button = IPA.action_button({
+            that.remove_button = IPA.button_widget({
                 name: 'remove',
                 label: '@i18n:buttons.remove',
-                icon: 'remove-icon',
-                'class': 'action-button-disabled',
-                click: function() {
-                    if (!that.remove_button.hasClass('action-button-disabled')) {
-                        that.remove_handler();
-                    }
-                    return false;
-                }
-            }).appendTo(that.table.buttons);
+                icon: 'fa-trash-o',
+                enabled: false,
+                button_class: 'btn btn-link',
+                click: that.remove_handler
+            });
+            that.remove_button.create(that.table.buttons);
 
-            that.add_button = IPA.action_button({
+            that.add_button = IPA.button_widget({
                 name: 'add',
                 label: '@i18n:buttons.add',
-                icon: 'add-icon',
-                click: function() {
-                    if (!that.add_button.hasClass('action-button-disabled')) {
-                        that.add_handler();
-                    }
-                    return false;
-                }
-            }).appendTo(that.table.buttons);
+                icon: 'fa-plus',
+                button_class: 'btn btn-link',
+                click: that.add_handler
+            });
+            that.add_button.create(that.table.buttons);
         };
 
         that.table.select_changed = function() {
@@ -732,11 +735,7 @@ IPA.sudo.options_section = function(spec) {
             var values = that.table.get_selected_values();
 
             if (that.remove_button) {
-                if (values.length === 0) {
-                    that.remove_button.addClass('action-button-disabled');
-                } else {
-                    that.remove_button.removeClass('action-button-disabled');
-                }
+                that.remove_button.set_enabled(values.length > 0);
             }
         };
 
@@ -767,7 +766,7 @@ IPA.sudo.options_section = function(spec) {
             dialog.callback = function() {
                 that.show_add_dialog();
             };
-            dialog.open(that.container);
+            dialog.open();
 
         } else {
             that.show_add_dialog();
@@ -783,7 +782,7 @@ IPA.sudo.options_section = function(spec) {
             dialog.callback = function() {
                 that.show_remove_dialog();
             };
-            dialog.open(that.container);
+            dialog.open();
 
         } else {
             that.show_remove_dialog();
@@ -821,7 +820,7 @@ IPA.sudo.options_section = function(spec) {
 
                 var pkey = that.facet.get_pkey();
 
-                var command = IPA.command({
+                var command = rpc.command({
                     entity: 'sudorule',
                     method: 'add_option',
                     args: [pkey],
@@ -851,7 +850,7 @@ IPA.sudo.options_section = function(spec) {
             }
         });
 
-        dialog.open(that.container);
+        dialog.open();
     };
 
     that.show_remove_dialog = function() {
@@ -877,7 +876,7 @@ IPA.sudo.options_section = function(spec) {
 
         dialog.execute = function() {
 
-            var batch = IPA.batch_command({
+            var batch = rpc.batch_command({
                 on_success: function(data) {
                     //last successful result of batch results contains valid data
                     var result;
@@ -904,7 +903,7 @@ IPA.sudo.options_section = function(spec) {
             });
 
             for (var i=0; i<values.length; i++) {
-                var command = IPA.command({
+                var command = rpc.command({
                     entity: 'sudorule',
                     method: 'remove_option',
                     args: [pkey]
@@ -918,11 +917,11 @@ IPA.sudo.options_section = function(spec) {
             batch.execute();
         };
 
-        dialog.open(that.container);
+        dialog.open();
     };
 
     that.reload = function() {
-        var command = IPA.command({
+        var command = rpc.command({
             entity: that.facet.entity.name,
             method: 'show',
             args: that.facet.get_pkeys(),
