@@ -50,7 +50,7 @@ def setup_package():
     try:
         setup(
             name = "ipaclient",
-            version = "4.3.1",
+            version = "4.3.2",
             license = "GPL",
             url = "http://www.freeipa.org/",
             description = DOCLINES[0],
@@ -61,6 +61,8 @@ def setup_package():
             package_dir = {'ipaclient': ''},
             packages = ["ipaclient",
                        ],
+            scripts=['../ipa'],
+            data_files = [('share/man/man1', ["../ipa.1"])],
         )
     finally:
         del sys.path[0]
