@@ -25,6 +25,7 @@ Contains Red Hat OS family-specific service class implementations.
 import os
 import time
 import xml.dom.minidom
+import contextlib
 
 from ipaplatform.tasks import tasks
 from ipaplatform.base import services as base_services
@@ -57,11 +58,6 @@ redhat_system_units['rpcidmapd'] = 'nfs-idmap.service'
 # code).
 
 redhat_system_units['dirsrv'] = 'dirsrv@.service'
-# Our directory server instance for PKI is dirsrv@PKI-IPA.service
-redhat_system_units['pkids'] = 'dirsrv@PKI-IPA.service'
-# Old style PKI instance
-redhat_system_units['pki-cad'] = 'pki-cad@pki-ca.service'
-redhat_system_units['pki_cad'] = redhat_system_units['pki-cad']
 # Our PKI instance is pki-tomcatd@pki-tomcat.service
 redhat_system_units['pki-tomcatd'] = 'pki-tomcatd@pki-tomcat.service'
 redhat_system_units['pki_tomcatd'] = redhat_system_units['pki-tomcatd']
@@ -124,7 +120,8 @@ class RedHatDirectoryService(RedHatService):
 
         return True
 
-    def restart(self, instance_name="", capture_output=True, wait=True):
+    def restart(self, instance_name="", capture_output=True, wait=True,
+                ldapi=False):
     # We need to explicitly enable instances to install proper symlinks as
     # dirsrv.target.wants/ dependencies. Standard systemd service class does it
     # on enable() method call. Unfortunately, ipa-server-install does not do
@@ -150,8 +147,35 @@ class RedHatDirectoryService(RedHatService):
                 os.unlink(srv_lnk)
                 os.symlink(srv_etc, srv_lnk)
 
-        super(RedHatDirectoryService, self).restart(instance_name,
-            capture_output=capture_output, wait=wait)
+        with self.__wait(instance_name, wait, ldapi) as wait:
+            super(RedHatDirectoryService, self).restart(
+                instance_name, capture_output=capture_output, wait=wait)
+
+    def start(self, instance_name="", capture_output=True, wait=True,
+              ldapi=False):
+        with self.__wait(instance_name, wait, ldapi) as wait:
+            super(RedHatDirectoryService, self).start(
+                instance_name, capture_output=capture_output, wait=wait)
+
+    @contextlib.contextmanager
+    def __wait(self, instance_name, wait, ldapi):
+        if ldapi:
+            instance_name = self.service_instance(instance_name)
+            if instance_name.endswith('.service'):
+                instance_name = instance_name[:-8]
+            if instance_name.startswith('dirsrv'):
+                # this is intentional, return the empty string if the instance
+                # name is 'dirsrv'
+                instance_name = instance_name[7:]
+            if not instance_name:
+                ldapi = False
+        if ldapi:
+            yield False
+            socket_name = paths.SLAPD_INSTANCE_SOCKET_TEMPLATE % instance_name
+            ipautil.wait_for_open_socket(socket_name,
+                                         self.api.env.startup_timeout)
+        else:
+            yield wait
 
 
 class RedHatIPAService(RedHatService):
@@ -171,16 +195,6 @@ class RedHatSSHService(RedHatService):
 
 class RedHatCAService(RedHatService):
     def wait_until_running(self):
-        # We must not wait for the httpd proxy if httpd is not set up yet.
-        # Unfortunately, knownservices.httpd.is_installed() can return
-        # false positives, so check for existence of our configuration file.
-        # TODO: Use a cleaner solution
-        use_proxy = True
-        if not (os.path.exists('/etc/httpd/conf.d/ipa.conf') and
-                os.path.exists(paths.HTTPD_IPA_PKI_PROXY_CONF)):
-            root_logger.debug(
-                'The httpd proxy is not installed, wait on local port')
-            use_proxy = False
         root_logger.debug('Waiting until the CA is running')
         timeout = float(api.env.startup_timeout)
         op_timeout = time.time() + timeout
@@ -192,8 +206,6 @@ class RedHatCAService(RedHatService):
                 # status = dogtag.ca_status(use_proxy=use_proxy)
                 #
                 port = 8443
-                if use_proxy:
-                    port = 443
 
                 url = "https://%(host_port)s%(path)s" % {
                     "host_port": ipautil.format_netloc(api.env.ca_host, port),
@@ -201,16 +213,16 @@ class RedHatCAService(RedHatService):
                 }
 
                 args = [
-                    paths.BIN_WGET,
-                    '-S', '-O', '-',
-                    '--timeout=30',
-                    '--no-check-certificate',
+                    paths.BIN_CURL,
+                    '-o', '-',
+                    '--connect-timeout', '30',
+                    '-k',
                     url
                 ]
 
-                stdout, stderr, returncode = ipautil.run(args)
+                result = ipautil.run(args, capture_output=True)
 
-                status = dogtag._parse_ca_status(stdout)
+                status = dogtag._parse_ca_status(result.output)
                 # end of workaround
             except Exception as e:
                 status = 'check interrupted due to error: %s' % e
@@ -235,28 +247,6 @@ class RedHatCAService(RedHatService):
             self.wait_until_running()
 
 
-class RedHatNamedService(RedHatService):
-    def get_user_name(self):
-        return u'named'
-
-    def get_group_name(self):
-        return u'named'
-
-    def get_binary_path(self):
-        return paths.NAMED_PKCS11
-
-    def get_package_name(self):
-        return u"bind-pkcs11"
-
-
-class RedHatODSEnforcerdService(RedHatService):
-    def get_user_name(self):
-        return u'ods'
-
-    def get_group_name(self):
-        return u'ods'
-
-
 # Function that constructs proper Red Hat OS family-specific server classes for
 # services of specified name
 
@@ -267,12 +257,8 @@ def redhat_service_class_factory(name):
         return RedHatIPAService(name)
     if name == 'sshd':
         return RedHatSSHService(name)
-    if name in ('pki-cad', 'pki_cad', 'pki-tomcatd', 'pki_tomcatd'):
+    if name in ('pki-tomcatd', 'pki_tomcatd'):
         return RedHatCAService(name)
-    if name == 'named':
-        return RedHatNamedService(name)
-    if name in ('ods-enforcerd', 'ods_enforcerd'):
-        return RedHatODSEnforcerdService(name)
     return RedHatService(name)
 
 
