@@ -21,8 +21,11 @@ import re
 import time
 import datetime
 import email.utils
-from urllib2 import urlparse
 from calendar import timegm
+
+import six
+from six.moves.urllib.parse import urlparse
+
 from ipapython.ipa_log_manager import log_mgr
 
 '''
@@ -46,7 +49,7 @@ the relevant RFC's as well as actual practice in the field. However
 cookielib.py is tighly integrated with urllib2 and it's not possible
 to use most of the features of cookielib without simultaneously using
 urllib2. Unfortunataely we only use httplib because of our dependency
-on xmlrpclib. Without urllib2 cookielib is a non-starter.
+on xmlrpc.client. Without urllib2 cookielib is a non-starter.
 
 This module is a minimal implementation of Netscape cookies which
 works equally well on either the client or server side. It's API is
@@ -185,7 +188,7 @@ class Cookie(object):
 
         try:
             dt = datetime.datetime(*email.utils.parsedate(s)[0:6])
-        except Exception, e:
+        except Exception as e:
             raise ValueError("unable to parse expires datetime '%s': %s" % (s, e))
 
         return dt
@@ -270,8 +273,9 @@ class Cookie(object):
             if match:
                 key = match.group(1)
                 value = match.group(2)
+
                 # Double quoted value?
-                if value[0] == '"':
+                if value and value[0] == '"':
                     if value[-1] == '"':
                         value = value[1:-1]
                     else:
@@ -388,9 +392,9 @@ class Cookie(object):
             self._timestamp = None
         elif isinstance(value, datetime.datetime):
             self._timestamp = value
-        elif isinstance(value, (int, long, float)):
+        elif isinstance(value, (six.integer_types, float)):
             self._timestamp = datetime.datetime.utcfromtimestamp(value)
-        elif isinstance(value, basestring):
+        elif isinstance(value, six.string_types):
             self._timestamp = Cookie.parse_datetime(value)
         else:
             raise TypeError('value must be datetime, int, long, float, basestring or None, not %s' % \
@@ -414,9 +418,9 @@ class Cookie(object):
             self._expires = None
         elif isinstance(value, datetime.datetime):
             self._expires = value
-        elif isinstance(value, (int, long, float)):
+        elif isinstance(value, (six.integer_types, float)):
             self._expires = datetime.datetime.utcfromtimestamp(value)
-        elif isinstance(value, basestring):
+        elif isinstance(value, six.string_types):
             self._expires = Cookie.parse_datetime(value)
         else:
             raise TypeError('value must be datetime, int, long, float, basestring or None, not %s' % \
@@ -537,7 +541,7 @@ class Cookie(object):
         received from.
         '''
 
-        scheme, domain, path, params, query, fragment = urlparse.urlparse(url)
+        scheme, domain, path, params, query, fragment = urlparse(url)
 
         if self.domain is None:
             self.domain = domain.lower()
@@ -594,7 +598,7 @@ class Cookie(object):
             from ipalib.util import validate_domain_name
             try:
                 validate_domain_name(url_domain)
-            except Exception, e:
+            except Exception as e:
                 return False
 
             if cookie_domain is None:
@@ -639,7 +643,7 @@ class Cookie(object):
 
         cookie_name = self.key
 
-        url_scheme, url_domain, url_path, url_params, url_query, url_fragment = urlparse.urlparse(url)
+        url_scheme, url_domain, url_path, url_params, url_query, url_fragment = urlparse(url)
 
         cookie_expiration = self.get_expiration()
         if cookie_expiration is not None:
