@@ -24,16 +24,19 @@ Also see the `ipalib.rpc` module.
 """
 
 from xml.sax.saxutils import escape
-from xmlrpclib import Fault
+from six.moves.xmlrpc_client import Fault
 import os
 import datetime
-import urlparse
 import json
 import traceback
+import gssapi
+import time
 
 import ldap.controls
 from pyasn1.type import univ, namedtype
 from pyasn1.codec.ber import encoder
+import six
+from six.moves.urllib.parse import parse_qs
 
 from ipalib import plugable, errors
 from ipalib.capabilities import VERSION_WITHOUT_CAPABILITIES
@@ -53,13 +56,16 @@ from ipalib.session import (
     default_max_session_duration, krbccache_dir, krbccache_prefix)
 from ipalib.backend import Backend
 from ipalib.krb_utils import (
-    KRB5_CCache, krb_ticket_expiration_threshold, krb5_format_principal_name,
-    krb5_format_service_principal_name)
+    krb_ticket_expiration_threshold, krb5_format_principal_name,
+    krb5_format_service_principal_name, get_credentials, get_credentials_if_valid)
 from ipapython import ipautil
 from ipaplatform.paths import paths
 from ipapython.version import VERSION
 from ipalib.text import _
 
+if six.PY3:
+    unicode = str
+
 HTTP_STATUS_SUCCESS = '200 Success'
 HTTP_STATUS_SERVER_ERROR = '500 Internal Server Error'
 
@@ -199,7 +205,7 @@ def params_2_args_options(params):
 def nicify_query(query, encoding='utf-8'):
     if not query:
         return
-    for (key, value) in query.iteritems():
+    for (key, value) in query.items():
         if len(value) == 0:
             yield (key, None)
         elif len(value) == 1:
@@ -219,9 +225,7 @@ def extract_query(environ):
     elif environ['REQUEST_METHOD'] == 'GET':
         qstr = environ['QUERY_STRING']
     if qstr:
-        query = dict(nicify_query(
-            urlparse.parse_qs(qstr)#, keep_blank_values=True)
-        ))
+        query = dict(nicify_query(parse_qs(qstr)))  # keep_blank_values=True)
     else:
         query = {}
     environ['wsgi.query'] = query
@@ -237,8 +241,8 @@ class wsgi_dispatch(Executioner, HTTP_Status):
     handler which is specific to the authentication and RPC mechanism.
     """
 
-    def __init__(self):
-        super(wsgi_dispatch, self).__init__()
+    def __init__(self, api):
+        super(wsgi_dispatch, self).__init__(api)
         self.__apps = {}
 
     def __iter__(self):
@@ -276,11 +280,11 @@ class wsgi_dispatch(Executioner, HTTP_Status):
         Mount the WSGI application *app* at *key*.
         """
 #        if self.__islocked__():
-#            raise StandardError('%s.mount(): locked, cannot mount %r at %r' % (
+#            raise Exception('%s.mount(): locked, cannot mount %r at %r' % (
 #                self.name, app, key)
 #            )
         if key in self.__apps:
-            raise StandardError('%s.mount(): cannot replace %r with %r at %r' % (
+            raise Exception('%s.mount(): cannot replace %r with %r at %r' % (
                 self.name, self.__apps[key], app, key)
             )
         self.debug('Mounting %r at %r', app, key)
@@ -300,14 +304,11 @@ class WSGIExecutioner(Executioner):
 
     _system_commands = {}
 
-    def set_api(self, api):
-        super(WSGIExecutioner, self).set_api(api)
-        if 'wsgi_dispatch' in self.api.Backend:
-            self.api.Backend.wsgi_dispatch.mount(self, self.key)
-
     def _on_finalize(self):
         self.url = self.env.mount_ipa + self.key
         super(WSGIExecutioner, self)._on_finalize()
+        if 'wsgi_dispatch' in self.api.Backend:
+            self.api.Backend.wsgi_dispatch.mount(self, self.key)
 
     def wsgi_execute(self, environ):
         result = None
@@ -347,11 +348,11 @@ class WSGIExecutioner(Executioner):
                 raise CommandError(name=name)
             else:
                 result = self.Command[name](*args, **options)
-        except PublicError, e:
+        except PublicError as e:
             if self.api.env.debug:
                 self.debug('WSGI wsgi_execute PublicError: %s', traceback.format_exc())
             error = e
-        except StandardError, e:
+        except Exception as e:
             self.exception(
                 'non-public: %s: %s', e.__class__.__name__, str(e)
             )
@@ -363,7 +364,7 @@ class WSGIExecutioner(Executioner):
         if name and name in self.Command:
             try:
                 params = self.Command[name].args_options_2_params(*args, **options)
-            except Exception, e:
+            except Exception as e:
                 self.info(
                    'exception %s caught when converting options: %s', e.__class__.__name__, str(e)
                 )
@@ -404,7 +405,7 @@ class WSGIExecutioner(Executioner):
             status = HTTP_STATUS_SUCCESS
             response = self.wsgi_execute(environ)
             headers = [('Content-Type', self.content_type + '; charset=utf-8')]
-        except StandardError, e:
+        except Exception as e:
             self.exception('WSGI %s.__call__():', self.name)
             status = HTTP_STATUS_SERVER_ERROR
             response = status
@@ -472,7 +473,7 @@ class jsonserver(WSGIExecutioner, HTTP_Status):
     def unmarshal(self, data):
         try:
             d = json.loads(data)
-        except ValueError, e:
+        except ValueError as e:
             raise JSONError(error=e)
         if not isinstance(d, dict):
             raise JSONError(error=_('Request must be a dict'))
@@ -494,7 +495,7 @@ class jsonserver(WSGIExecutioner, HTTP_Status):
         options = params[1]
         if not isinstance(options, dict):
             raise JSONError(error=_('params[1] (aka options) must be a dict'))
-        options = dict((str(k), v) for (k, v) in options.iteritems())
+        options = dict((str(k), v) for (k, v) in options.items())
         return (method, args, options, _id)
 
 class AuthManagerKerb(AuthManager):
@@ -519,7 +520,7 @@ class AuthManagerKerb(AuthManager):
         of the login mechanisms.
         '''
 
-        if session_data.has_key('ccache_data'):
+        if 'ccache_data' in session_data:
             self.debug('AuthManager.logout.%s: deleting ccache_data', self.name)
             del session_data['ccache_data']
         else:
@@ -550,7 +551,7 @@ class KerberosSession(object):
             seconds = parse_time_duration(self.api.env.session_auth_duration)
             self.session_auth_duration = int(seconds)
             self.debug("session_auth_duration: %s", datetime.timedelta(seconds=self.session_auth_duration))
-        except Exception, e:
+        except Exception as e:
             self.session_auth_duration = default_max_session_duration
             self.error('unable to parse session_auth_duration, defaulting to %d: %s',
                        self.session_auth_duration, e)
@@ -595,8 +596,8 @@ class KerberosSession(object):
         session_data['ccache_data'] = load_ccache_data(ccache_name)
 
         # Set when the session will expire
-        cc = KRB5_CCache(ccache_name)
-        endtime = cc.endtime(self.api.env.host, self.api.env.realm)
+        creds = get_credentials(ccache_name=ccache_name)
+        endtime = creds.lifetime + time.time()
         self.update_session_expiration(session_data, endtime)
 
         # Store the session data now that it's been updated with the ccache
@@ -647,7 +648,7 @@ class KerberosWSGIExecutioner(WSGIExecutioner, HTTP_Status, KerberosSession):
             if (session_data is None and self.env.context != 'lite'):
                 self.finalize_kerberos_acquisition(
                     'xmlserver', user_ccache, environ, start_response, headers)
-        except PublicError, e:
+        except PublicError as e:
             status = HTTP_STATUS_SUCCESS
             response = status
             start_response(status, headers)
@@ -745,9 +746,10 @@ class jsonserver_session(jsonserver, KerberosSession):
 
     key = '/session/json'
 
-    def __init__(self):
-        super(jsonserver_session, self).__init__()
-        auth_mgr = AuthManagerKerb(self.__class__.__name__)
+    def __init__(self, api):
+        super(jsonserver_session, self).__init__(api)
+        name = '{0}_{1}'.format(self.__class__.__name__, id(self))
+        auth_mgr = AuthManagerKerb(name)
         session_mgr.auth_mgr.register(auth_mgr.name, auth_mgr)
 
     def _on_finalize(self):
@@ -790,15 +792,15 @@ class jsonserver_session(jsonserver, KerberosSession):
         ipa_ccache_name = bind_ipa_ccache(ccache_data)
 
         # Redirect to login if Kerberos credentials are expired
-        cc = KRB5_CCache(ipa_ccache_name)
-        if not cc.valid(self.api.env.host, self.api.env.realm):
+        creds = get_credentials_if_valid(ccache_name=ipa_ccache_name)
+        if not creds:
             self.debug('ccache expired, deleting session, need login')
             # The request is finished with the ccache, destroy it.
             release_ipa_ccache(ipa_ccache_name)
             return self.need_login(start_response)
 
         # Update the session expiration based on the Kerberos expiration
-        endtime = cc.endtime(self.api.env.host, self.api.env.realm)
+        endtime = creds.lifetime + time.time()
         self.update_session_expiration(session_data, endtime)
 
         # Store the session data in the per-thread context
@@ -807,7 +809,7 @@ class jsonserver_session(jsonserver, KerberosSession):
         # This may fail if a ticket from wrong realm was handled via browser
         try:
             self.create_context(ccache=ipa_ccache_name)
-        except ACIError, e:
+        except ACIError as e:
             return self.unauthorized(environ, start_response, str(e), 'denied')
 
         try:
@@ -824,7 +826,7 @@ class jsonserver_session(jsonserver, KerberosSession):
             # logout command removes the ccache data from the session
             # data to invalidate the session credentials.
 
-            if session_data.has_key('ccache_data'):
+            if 'ccache_data' in session_data:
                 session_data['ccache_data'] = load_ccache_data(ipa_ccache_name)
 
             # The request is finished with the ccache, destroy it.
@@ -847,9 +849,6 @@ class jsonserver_kerb(jsonserver, KerberosWSGIExecutioner):
 class login_kerberos(Backend, KerberosSession, HTTP_Status):
     key = '/session/login_kerberos'
 
-    def __init__(self):
-        super(login_kerberos, self).__init__()
-
     def _on_finalize(self):
         super(login_kerberos, self)._on_finalize()
         self.api.Backend.wsgi_dispatch.mount(self, self.key)
@@ -858,7 +857,7 @@ class login_kerberos(Backend, KerberosSession, HTTP_Status):
     def __call__(self, environ, start_response):
         self.debug('WSGI login_kerberos.__call__:')
 
-        # Get the ccache created by mod_auth_kerb
+        # Get the ccache created by mod_auth_gssapi
         user_ccache_name=environ.get('KRB5CCNAME')
         if user_ccache_name is None:
             return self.internal_error(environ, start_response,
@@ -871,9 +870,6 @@ class login_password(Backend, KerberosSession, HTTP_Status):
     content_type = 'text/plain'
     key = '/session/login_password'
 
-    def __init__(self):
-        super(login_password, self).__init__()
-
     def _on_finalize(self):
         super(login_password, self)._on_finalize()
         self.api.Backend.wsgi_dispatch.mount(self, self.key)
@@ -894,8 +890,8 @@ class login_password(Backend, KerberosSession, HTTP_Status):
             return self.bad_request(environ, start_response, "HTTP request method must be POST")
 
         try:
-            query_dict = urlparse.parse_qs(query_string)
-        except Exception, e:
+            query_dict = parse_qs(query_string)
+        except Exception as e:
             return self.bad_request(environ, start_response, "cannot parse query data")
 
         user = query_dict.get('user', None)
@@ -958,8 +954,8 @@ class login_password(Backend, KerberosSession, HTTP_Status):
 
     def kinit(self, user, realm, password, ccache_name):
         # get http service ccache as an armor for FAST to enable OTP authentication
-        armor_principal = krb5_format_service_principal_name(
-            'HTTP', self.api.env.host, realm)
+        armor_principal = str(krb5_format_service_principal_name(
+            'HTTP', self.api.env.host, realm))
         keytab = paths.IPA_KEYTAB
         armor_name = "%sA_%s" % (krbccache_prefix, user)
         armor_path = os.path.join(krbccache_dir, armor_name)
@@ -967,43 +963,35 @@ class login_password(Backend, KerberosSession, HTTP_Status):
         self.debug('Obtaining armor ccache: principal=%s keytab=%s ccache=%s',
                    armor_principal, keytab, armor_path)
 
-        (stdout, stderr, returncode) = ipautil.run(
-            [paths.KINIT, '-kt', keytab, armor_principal],
-            env={'KRB5CCNAME': armor_path}, raiseonerr=False)
-
-        if returncode != 0:
-            raise CCacheError()
+        try:
+            ipautil.kinit_keytab(armor_principal, paths.IPA_KEYTAB, armor_path)
+        except gssapi.exceptions.GSSError as e:
+            raise CCacheError(message=unicode(e))
 
         # Format the user as a kerberos principal
         principal = krb5_format_principal_name(user, realm)
 
-        (stdout, stderr, returncode) = ipautil.run(
-            [paths.KINIT, principal, '-T', armor_path],
-            env={'KRB5CCNAME': ccache_name, 'LC_ALL': 'C'},
-            stdin=password, raiseonerr=False)
+        try:
+            ipautil.kinit_password(principal, password, ccache_name,
+                                   armor_ccache_name=armor_path)
 
-        self.debug('kinit: principal=%s returncode=%s, stderr="%s"',
-                   principal, returncode, stderr)
-
-        self.debug('Cleanup the armor ccache')
-        ipautil.run(
-            [paths.KDESTROY, '-A', '-c', armor_path],
-            env={'KRB5CCNAME': armor_path},
-            raiseonerr=False)
-
-        if returncode != 0:
-            if stderr.strip() == 'kinit: Cannot read password while getting initial credentials':
-                raise PasswordExpired(principal=principal, message=unicode(stderr))
-            raise InvalidSessionPassword(principal=principal, message=unicode(stderr))
+            self.debug('Cleanup the armor ccache')
+            ipautil.run(
+                [paths.KDESTROY, '-A', '-c', armor_path],
+                env={'KRB5CCNAME': armor_path},
+                raiseonerr=False)
+        except RuntimeError as e:
+            if ('kinit: Cannot read password while '
+                    'getting initial credentials') in str(e):
+                raise PasswordExpired(principal=principal, message=unicode(e))
+            raise InvalidSessionPassword(principal=principal,
+                                         message=unicode(e))
 
 class change_password(Backend, HTTP_Status):
 
     content_type = 'text/plain'
     key = '/session/change_password'
 
-    def __init__(self):
-        super(change_password, self).__init__()
-
     def _on_finalize(self):
         super(change_password, self)._on_finalize()
         self.api.Backend.wsgi_dispatch.mount(self, self.key)
@@ -1023,8 +1011,8 @@ class change_password(Backend, HTTP_Status):
             return self.bad_request(environ, start_response, "HTTP request method must be POST")
 
         try:
-            query_dict = urlparse.parse_qs(query_string)
-        except Exception, e:
+            query_dict = parse_qs(query_string)
+        except Exception as e:
             return self.bad_request(environ, start_response, "cannot parse query data")
 
         data = {}
@@ -1054,24 +1042,23 @@ class change_password(Backend, HTTP_Status):
             pw = data['old_password']
             if data.get('otp'):
                 pw = data['old_password'] + data['otp']
-            conn = ldap2(shared_instance=False,
-                         ldap_uri=self.api.env.ldap_uri)
+            conn = ldap2(self.api)
             conn.connect(bind_dn=bind_dn, bind_pw=pw)
         except (NotFound, ACIError):
             result = 'invalid-password'
             message = 'The old password or username is not correct.'
-        except Exception, e:
+        except Exception as e:
             message = "Could not connect to LDAP server."
             self.error("change_password: cannot authenticate '%s' to LDAP server: %s",
                     data['user'], str(e))
         else:
             try:
                 conn.modify_password(bind_dn, data['new_password'], data['old_password'], skip_bind=True)
-            except ExecutionError, e:
+            except ExecutionError as e:
                 result = 'policy-error'
                 policy_error = escape(str(e))
                 message = "Password change was rejected: %s" % escape(str(e))
-            except Exception, e:
+            except Exception as e:
                 message = "Could not change the password"
                 self.error("change_password: cannot change password of '%s': %s",
                         data['user'], str(e))
@@ -1081,7 +1068,7 @@ class change_password(Backend, HTTP_Status):
                 message = "Password was changed."
             finally:
                 if conn.isconnected():
-                    conn.destroy_connection()
+                    conn.disconnect()
 
         self.info('%s: %s', status, message)
 
@@ -1107,9 +1094,6 @@ class sync_token(Backend, HTTP_Status):
             namedtype.OptionalNamedType('tokenDN', univ.OctetString())
         )
 
-    def __init__(self):
-        super(sync_token, self).__init__()
-
     def _on_finalize(self):
         super(sync_token, self)._on_finalize()
         self.api.Backend.wsgi_dispatch.mount(self, self.key)
@@ -1129,8 +1113,8 @@ class sync_token(Backend, HTTP_Status):
 
         # Parse the query string to a dictionary.
         try:
-            query_dict = urlparse.parse_qs(query_string)
-        except Exception, e:
+            query_dict = parse_qs(query_string)
+        except Exception as e:
             return self.bad_request(environ, start_response, "cannot parse query data")
         data = {}
         for field in ('user', 'password', 'first_code', 'second_code', 'token'):
@@ -1168,7 +1152,7 @@ class sync_token(Backend, HTTP_Status):
         title = 'Token sync rejected'
 
         # Perform the synchronization.
-        conn = ldap2(shared_instance=False, ldap_uri=self.api.env.ldap_uri)
+        conn = ldap2(self.api)
         try:
             conn.connect(bind_dn=bind_dn,
                          bind_pw=data['password'],
@@ -1179,14 +1163,14 @@ class sync_token(Backend, HTTP_Status):
         except (NotFound, ACIError):
             result = 'invalid-credentials'
             message = 'The username, password or token codes are not correct.'
-        except Exception, e:
+        except Exception as e:
             result = 'error'
             message = "Could not connect to LDAP server."
             self.error("token_sync: cannot authenticate '%s' to LDAP server: %s",
                        data['user'], str(e))
         finally:
             if conn.isconnected():
-                conn.destroy_connection()
+                conn.disconnect()
 
         # Report status and return.
         response_headers.append(('X-IPA-TokenSync-Result', result))
@@ -1202,9 +1186,10 @@ class xmlserver_session(xmlserver, KerberosSession):
 
     key = '/session/xml'
 
-    def __init__(self):
-        super(xmlserver_session, self).__init__()
-        auth_mgr = AuthManagerKerb(self.__class__.__name__)
+    def __init__(self, api):
+        super(xmlserver_session, self).__init__(api)
+        name = '{0}_{1}'.format(self.__class__.__name__, id(self))
+        auth_mgr = AuthManagerKerb(name)
         session_mgr.auth_mgr.register(auth_mgr.name, auth_mgr)
 
     def _on_finalize(self):
@@ -1247,15 +1232,15 @@ class xmlserver_session(xmlserver, KerberosSession):
         ipa_ccache_name = bind_ipa_ccache(ccache_data)
 
         # Redirect to /ipa/xml if Kerberos credentials are expired
-        cc = KRB5_CCache(ipa_ccache_name)
-        if not cc.valid(self.api.env.host, self.api.env.realm):
+        creds = get_credentials_if_valid(ccache_name=ipa_ccache_name)
+        if not creds:
             self.debug('xmlserver_session.__call_: ccache expired, deleting session, need login')
             # The request is finished with the ccache, destroy it.
             release_ipa_ccache(ipa_ccache_name)
             return self.need_login(start_response)
 
         # Update the session expiration based on the Kerberos expiration
-        endtime = cc.endtime(self.api.env.host, self.api.env.realm)
+        endtime = creds.lifetime + time.time()
         self.update_session_expiration(session_data, endtime)
 
         # Store the session data in the per-thread context
@@ -1277,7 +1262,7 @@ class xmlserver_session(xmlserver, KerberosSession):
             # logout command removes the ccache data from the session
             # data to invalidate the session credentials.
 
-            if session_data.has_key('ccache_data'):
+            if 'ccache_data' in session_data:
                 session_data['ccache_data'] = load_ccache_data(ipa_ccache_name)
 
             # The request is finished with the ccache, destroy it.
