@@ -22,8 +22,12 @@ Test the `ipalib/plugins/config.py` module.
 """
 
 from ipalib import errors
-from xmlrpc_test import Declarative, fuzzy_digits, fuzzy_uuid
+from ipatests.test_xmlrpc.xmlrpc_test import (Declarative, fuzzy_digits,
+                                              fuzzy_uuid)
+import pytest
 
+
+@pytest.mark.tier1
 class test_config(Declarative):
 
     cleanup_commands = [
@@ -37,7 +41,7 @@ class test_config(Declarative):
                 dict(addattr=u'ipauserobjectclasses=ipahost')),
             expected=dict(
                     result=lambda d: 'ipahost' in d['ipauserobjectclasses'],
-                    value=u'',
+                    value=None,
                     summary=None,
                 ),
         ),
@@ -48,7 +52,7 @@ class test_config(Declarative):
                 dict(delattr=u'ipauserobjectclasses=ipahost')),
             expected=dict(
                     result=lambda d: 'ipahost' not in d['ipauserobjectclasses'],
-                    value=u'',
+                    value=None,
                     summary=None,
                 ),
         ),
@@ -118,4 +122,34 @@ class test_config(Declarative):
                 error='SELinux user map default user not in order list'),
         ),
 
+        dict(
+            desc='Set user auth type',
+            command=('config_mod', [], dict(ipauserauthtype=u'password')),
+            expected=dict(
+                    result=lambda d: d['ipauserauthtype'] == (u'password',),
+                    value=None,
+                    summary=None,
+                ),
+        ),
+
+        dict(
+            desc='Check user auth type',
+            command=('config_show', [], {}),
+            expected=dict(
+                    result=lambda d: d['ipauserauthtype'] == (u'password',),
+                    value=None,
+                    summary=None,
+                ),
+        ),
+
+        dict(
+            desc='Unset user auth type',
+            command=('config_mod', [], dict(ipauserauthtype=None)),
+            expected=dict(
+                    result=lambda d: 'ipauserauthtype' not in d,
+                    value=None,
+                    summary=None,
+                ),
+        ),
+
     ]
