@@ -525,13 +525,6 @@ class test_old_permission(Declarative):
                         'subtree': u'ldap:///%s' % users_dn,
                     },
                 ],
-                messages=({
-                    'message': (u'Search result has been truncated to '
-                                'configured search limit.'),
-                    'code': 13017,
-                    'type': u'warning',
-                    'name': u'SearchResultTruncated'
-                },),
             ),
         ),
 
@@ -585,13 +578,6 @@ class test_old_permission(Declarative):
                     DN(res['dn']).endswith(DN(api.env.container_permission,
                                               api.env.basedn)) and
                     'ipapermission' in res['objectclass']],
-                messages=({
-                    'message': (u'Search result has been truncated to '
-                                'configured search limit.'),
-                    'code': 13017,
-                    'type': u'warning',
-                    'name': u'SearchResultTruncated'
-                },),
             ),
         ),
 
