@@ -33,9 +33,9 @@ from ipalib import api, errors
 from ipatests.test_xmlrpc import objectclasses
 from ipatests.util import (
     assert_deepequal, assert_equal, assert_not_equal, raises)
-from xmlrpc_test import (
+from ipatests.test_xmlrpc.xmlrpc_test import (
     XMLRPC_test, fuzzy_digits, fuzzy_uuid, fuzzy_password,
-    fuzzy_string, fuzzy_dergeneralizedtime, add_sid, add_oc, raises_exact)
+    Fuzzy, fuzzy_dergeneralizedtime, add_sid, add_oc, raises_exact)
 from ipapython.dn import DN
 
 from ipatests.test_xmlrpc.tracker.base import Tracker
@@ -483,7 +483,7 @@ class TestCreate(XMLRPC_test):
         testuser.attrs.update(
             randompassword=fuzzy_password,
             has_keytab=True, has_password=True,
-            krbextradata=[fuzzy_string],
+            krbextradata=[Fuzzy(type=bytes)],
             krbpasswordexpiration=[fuzzy_dergeneralizedtime],
             krblastpwdchange=[fuzzy_dergeneralizedtime]
         )
