@@ -18,11 +18,14 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * This file includes an "OpenSSL license exception", see the
+ * COPYING.openssl file for details.
+ *
  */
 
 #include <stdbool.h>
 #include <iconv.h>
-#include <openssl/des.h>
 #include <openssl/md4.h>
 #include <dirsrv/slapi-plugin.h>
 
